@@ -701,8 +701,16 @@ class S3Product(OpticalProduct):
                 with rioxarray.open_rasterio(
                     str(rasters.get_dim_img_path(out_dim, snap_band_name))
                 ) as arr:
-                    arr = arr.where(arr != self._snap_no_data)
-                    rasters.write(arr, out_tif, dtype=np.float32)
+                    if arr.dtype == np.uint8:
+                        nodata_val = 255  # Exceptions
+                    elif arr.dtype == np.uint16:
+                        nodata_val = 65535  # Clouds
+                    elif arr.dtype == np.int32:
+                        nodata_val = 2147483647  # Quality flags
+                    else:
+                        nodata_val = self._snap_no_data
+                    arr = arr.where(arr != nodata_val)
+                    rasters.write(arr, out_tif)
 
         # Get the wanted bands (not the quality flags here !)
         for band in processed_bands:
@@ -1035,11 +1043,13 @@ class S3Product(OpticalProduct):
 
             try:
                 cloud_path = files.get_file_in_dir(
-                    self._get_band_folder(), "cloud_RAD.tif"
+                    self._get_band_folder(), f"{self.condensed_name}_cloud_RAD.tif"
                 )
             except FileNotFoundError:
                 self._preprocess_s3(resolution)
-                cloud_path = files.get_file_in_dir(self._tmp_process, "cloud_RAD.tif")
+                cloud_path = files.get_file_in_dir(
+                    self._tmp_process, f"{self.condensed_name}_cloud_RAD.tif"
+                )
 
             if not cloud_path:
                 raise FileNotFoundError(
@@ -1056,8 +1066,7 @@ class S3Product(OpticalProduct):
             ).astype(np.uint16)
 
             # Get nodata mask
-            # nodata = np.where(np.isnan(clouds_array), 1, 0)
-            nodata = np.where(clouds_array == 65535, 1, 0)
+            nodata = np.where(np.isnan(clouds_array), 1, 0)
 
             for band in bands:
                 if band == ALL_CLOUDS:
